@@ -32,6 +32,7 @@
 #include "sanlock_resource.h"
 #include "sanlock_admin.h"
 #include "sanlock_sock.h"
+#include "sanlock_rv.h"
 
 #ifndef GNUC_UNUSED
 #define GNUC_UNUSED __attribute__((__unused__))
@@ -78,13 +79,37 @@ static int send_header(int sock, int cmd, uint32_t cmd_flags, int datalen,
 	header.data = data;
 	header.data2 = data2;
 
+retry:
 	rv = send(sock, (void *) &header, sizeof(header), 0);
+	if (rv == -1 && errno == EINTR)
+		goto retry;
+
 	if (rv < 0)
 		return -errno;
 
 	return 0;
 }
 
+static ssize_t send_data(int sockfd, const void *buf, size_t len, int flags)
+{
+	ssize_t rv;
+retry:
+	rv = send(sockfd, buf, len, flags);
+	if (rv == -1 && errno == EINTR)
+		goto retry;
+	return rv;
+}
+
+static ssize_t recv_data(int sockfd, void *buf, size_t len, int flags)
+{
+	ssize_t rv;
+retry:
+	rv = recv(sockfd, buf, len, flags);
+	if (rv == -1 && errno == EINTR)
+		goto retry;
+	return rv;
+}
+
 int send_command(int cmd, uint32_t data);
 
 int send_command(int cmd, uint32_t data)
@@ -110,8 +135,10 @@ static int recv_result(int fd)
 	int rv;
 
 	memset(&h, 0, sizeof(h));
-
+retry:
 	rv = recv(fd, &h, sizeof(h), MSG_WAITALL);
+	if (rv == -1 && errno == EINTR)
+		goto retry;
 	if (rv < 0)
 		return -errno;
 	if (rv != sizeof(h))
@@ -132,7 +159,7 @@ static int cmd_lockspace(int cmd, struct sanlk_lockspace *ls, uint32_t flags, ui
 	if (rv < 0)
 		goto out;
 
-	rv = send(fd, (void *)ls, sizeof(struct sanlk_lockspace), 0);
+	rv = send_data(fd, (void *)ls, sizeof(struct sanlk_lockspace), 0);
 	if (rv < 0) {
 		rv = -errno;
 		goto out;
@@ -183,7 +210,7 @@ int sanlock_get_lockspaces(struct sanlk_lockspace **lss, int *lss_count,
 
 	memset(&h, 0, sizeof(h));
 
-	rv = recv(fd, &h, sizeof(h), MSG_WAITALL);
+	rv = recv_data(fd, &h, sizeof(h), MSG_WAITALL);
 	if (rv < 0) {
 		rv = -errno;
 		goto out;
@@ -213,7 +240,7 @@ int sanlock_get_lockspaces(struct sanlk_lockspace **lss, int *lss_count,
 	ls = lsbuf;
 
 	for (i = 0; i < recv_count; i++) {
-		ret = recv(fd, ls, sizeof(struct sanlk_lockspace), MSG_WAITALL);
+		ret = recv_data(fd, ls, sizeof(struct sanlk_lockspace), MSG_WAITALL);
 		if (ret < 0) {
 			rv = -errno;
 			free(lsbuf);
@@ -261,7 +288,7 @@ int sanlock_get_hosts(const char *ls_name, uint64_t host_id,
 	if (rv < 0)
 		goto out;
 
-	rv = send(fd, &ls, sizeof(struct sanlk_lockspace), 0);
+	rv = send_data(fd, &ls, sizeof(struct sanlk_lockspace), 0);
 	if (rv < 0) {
 		rv = -errno;
 		goto out;
@@ -271,7 +298,7 @@ int sanlock_get_hosts(const char *ls_name, uint64_t host_id,
 
 	memset(&h, 0, sizeof(h));
 
-	rv = recv(fd, &h, sizeof(h), MSG_WAITALL);
+	rv = recv_data(fd, &h, sizeof(h), MSG_WAITALL);
 	if (rv < 0) {
 		rv = -errno;
 		goto out;
@@ -301,7 +328,7 @@ int sanlock_get_hosts(const char *ls_name, uint64_t host_id,
 	hs = hsbuf;
 
 	for (i = 0; i < recv_count; i++) {
-		ret = recv(fd, hs, sizeof(struct sanlk_host), MSG_WAITALL);
+		ret = recv_data(fd, hs, sizeof(struct sanlk_host), MSG_WAITALL);
 		if (ret < 0) {
 			rv = -errno;
 			free(hsbuf);
@@ -345,7 +372,7 @@ int sanlock_set_config(const char *ls_name, uint32_t flags, uint32_t cmd, GNUC_U
 	if (rv < 0)
 		goto out;
 
-	rv = send(fd, &ls, sizeof(ls), 0);
+	rv = send_data(fd, &ls, sizeof(ls), 0);
 	if (rv < 0) {
 		rv = -errno;
 		goto out;
@@ -353,7 +380,7 @@ int sanlock_set_config(const char *ls_name, uint32_t flags, uint32_t cmd, GNUC_U
 
 	memset(&h, 0, sizeof(h));
 
-	rv = recv(fd, &h, sizeof(h), MSG_WAITALL);
+	rv = recv_data(fd, &h, sizeof(h), MSG_WAITALL);
 	if (rv < 0) {
 		rv = -errno;
 		goto out;
@@ -382,7 +409,7 @@ int sanlock_align(struct sanlk_disk *disk)
 	if (rv < 0)
 		goto out;
 
-	rv = send(fd, (void *)disk, sizeof(struct sanlk_disk), 0);
+	rv = send_data(fd, (void *)disk, sizeof(struct sanlk_disk), 0);
 	if (rv < 0) {
 		rv = -errno;
 		goto out;
@@ -412,7 +439,7 @@ int sanlock_read_lockspace(struct sanlk_lockspace *ls, uint32_t flags, uint32_t
 	if (rv < 0)
 		goto out;
 
-	rv = send(fd, ls, sizeof(struct sanlk_lockspace), 0);
+	rv = send_data(fd, ls, sizeof(struct sanlk_lockspace), 0);
 	if (rv < 0) {
 		rv = -errno;
 		goto out;
@@ -422,7 +449,7 @@ int sanlock_read_lockspace(struct sanlk_lockspace *ls, uint32_t flags, uint32_t
 
 	memset(&h, 0, sizeof(h));
 
-	rv = recv(fd, &h, sizeof(h), MSG_WAITALL);
+	rv = recv_data(fd, &h, sizeof(h), MSG_WAITALL);
 	if (rv < 0) {
 		rv = -errno;
 		goto out;
@@ -437,7 +464,7 @@ int sanlock_read_lockspace(struct sanlk_lockspace *ls, uint32_t flags, uint32_t
 	if (rv < 0)
 		goto out;
 
-	rv = recv(fd, ls, sizeof(struct sanlk_lockspace), MSG_WAITALL);
+	rv = recv_data(fd, ls, sizeof(struct sanlk_lockspace), MSG_WAITALL);
 	if (rv < 0) {
 		rv = -errno;
 		goto out;
@@ -475,13 +502,13 @@ int sanlock_read_resource(struct sanlk_resource *res, uint32_t flags)
 	if (rv < 0)
 		goto out;
 
-	rv = send(fd, res, sizeof(struct sanlk_resource), 0);
+	rv = send_data(fd, res, sizeof(struct sanlk_resource), 0);
 	if (rv < 0) {
 		rv = -errno;
 		goto out;
 	}
 
-	rv = send(fd, res->disks, sizeof(struct sanlk_disk) * res->num_disks, 0);
+	rv = send_data(fd, res->disks, sizeof(struct sanlk_disk) * res->num_disks, 0);
 	if (rv < 0) {
 		rv = -errno;
 		goto out;
@@ -491,7 +518,7 @@ int sanlock_read_resource(struct sanlk_resource *res, uint32_t flags)
 
 	memset(&h, 0, sizeof(h));
 
-	rv = recv(fd, &h, sizeof(h), MSG_WAITALL);
+	rv = recv_data(fd, &h, sizeof(h), MSG_WAITALL);
 	if (rv < 0) {
 		rv = -errno;
 		goto out;
@@ -506,7 +533,7 @@ int sanlock_read_resource(struct sanlk_resource *res, uint32_t flags)
 	if (rv < 0)
 		goto out;
 
-	rv = recv(fd, res, sizeof(struct sanlk_resource), MSG_WAITALL);
+	rv = recv_data(fd, res, sizeof(struct sanlk_resource), MSG_WAITALL);
 	if (rv < 0) {
 		rv = -errno;
 		goto out;
@@ -541,7 +568,7 @@ int sanlock_write_lockspace(struct sanlk_lockspace *ls, int max_hosts,
 	if (rv < 0)
 		goto out;
 
-	rv = send(fd, ls, sizeof(struct sanlk_lockspace), 0);
+	rv = send_data(fd, ls, sizeof(struct sanlk_lockspace), 0);
 	if (rv < 0) {
 		rv = -errno;
 		goto out;
@@ -573,13 +600,13 @@ int sanlock_write_resource(struct sanlk_resource *res,
 	if (rv < 0)
 		goto out;
 
-	rv = send(fd, res, sizeof(struct sanlk_resource), 0);
+	rv = send_data(fd, res, sizeof(struct sanlk_resource), 0);
 	if (rv < 0) {
 		rv = -errno;
 		goto out;
 	}
 
-	rv = send(fd, res->disks, sizeof(struct sanlk_disk) * res->num_disks, 0);
+	rv = send_data(fd, res->disks, sizeof(struct sanlk_disk) * res->num_disks, 0);
 	if (rv < 0) {
 		rv = -errno;
 		goto out;
@@ -613,13 +640,13 @@ int sanlock_read_resource_owners(struct sanlk_resource *res, uint32_t flags,
 	if (rv < 0)
 		goto out;
 
-	rv = send(fd, res, sizeof(struct sanlk_resource), 0);
+	rv = send_data(fd, res, sizeof(struct sanlk_resource), 0);
 	if (rv < 0) {
 		rv = -errno;
 		goto out;
 	}
 
-	rv = send(fd, res->disks, sizeof(struct sanlk_disk) * res->num_disks, 0);
+	rv = send_data(fd, res->disks, sizeof(struct sanlk_disk) * res->num_disks, 0);
 	if (rv < 0) {
 		rv = -errno;
 		goto out;
@@ -629,7 +656,7 @@ int sanlock_read_resource_owners(struct sanlk_resource *res, uint32_t flags,
 
 	memset(&h, 0, sizeof(h));
 
-	rv = recv(fd, &h, sizeof(h), MSG_WAITALL);
+	rv = recv_data(fd, &h, sizeof(h), MSG_WAITALL);
 	if (rv < 0) {
 		rv = -errno;
 		goto out;
@@ -644,7 +671,7 @@ int sanlock_read_resource_owners(struct sanlk_resource *res, uint32_t flags,
 	if (rv < 0)
 		goto out;
 
-	rv = recv(fd, res, sizeof(struct sanlk_resource), MSG_WAITALL);
+	rv = recv_data(fd, res, sizeof(struct sanlk_resource), MSG_WAITALL);
 	if (rv < 0) {
 		rv = -errno;
 		goto out;
@@ -670,7 +697,7 @@ int sanlock_read_resource_owners(struct sanlk_resource *res, uint32_t flags,
 	hs = hsbuf;
 
 	for (i = 0; i < recv_count; i++) {
-		ret = recv(fd, hs, sizeof(struct sanlk_host), MSG_WAITALL);
+		ret = recv_data(fd, hs, sizeof(struct sanlk_host), MSG_WAITALL);
 		if (ret < 0) {
 			rv = -errno;
 			free(hsbuf);
@@ -774,17 +801,17 @@ int sanlock_reg_event(const char *ls_name, struct sanlk_host_event *he, uint32_t
 	if (rv < 0)
 		goto fail;
 
-	rv = send(reg_fd, &ls, sizeof(ls), 0);
+	rv = send_data(reg_fd, &ls, sizeof(ls), 0);
 	if (rv < 0)
 		goto fail;
 
-	rv = send(reg_fd, &ev, sizeof(ev), 0);
+	rv = send_data(reg_fd, &ev, sizeof(ev), 0);
 	if (rv < 0)
 		goto fail;
 
 	memset(&h, 0, sizeof(h));
 
-	rv = recv(reg_fd, &h, sizeof(h), MSG_WAITALL);
+	rv = recv_data(reg_fd, &h, sizeof(h), MSG_WAITALL);
 	if (rv < 0) {
 		rv = -errno;
 		goto fail;
@@ -821,7 +848,7 @@ int sanlock_end_event(int reg_fd, const char *ls_name, uint32_t flags)
 	 * check if they have been unregistered.
 	 */
 
-	rv = send(reg_fd, &end, sizeof(end), 0);
+	rv = send_data(reg_fd, &end, sizeof(end), 0);
 	if (rv < 0) {
 		close(reg_fd);
 		return -EALREADY;
@@ -848,13 +875,13 @@ int sanlock_end_event(int reg_fd, const char *ls_name, uint32_t flags)
 	if (rv < 0)
 		goto out;
 
-	rv = send(fd, &ls, sizeof(ls), 0);
+	rv = send_data(fd, &ls, sizeof(ls), 0);
 	if (rv < 0)
 		goto out;
 
 	memset(&h, 0, sizeof(h));
 
-	rv = recv(fd, &h, sizeof(h), MSG_WAITALL);
+	rv = recv_data(fd, &h, sizeof(h), MSG_WAITALL);
 	if (rv < 0) {
 		rv = -errno;
 		goto out;
@@ -897,13 +924,13 @@ int sanlock_set_event(const char *ls_name, struct sanlk_host_event *he, uint32_t
 	if (rv < 0)
 		goto out;
 
-	rv = send(fd, &ls, sizeof(ls), 0);
+	rv = send_data(fd, &ls, sizeof(ls), 0);
 	if (rv < 0) {
 		rv = -errno;
 		goto out;
 	}
 
-	rv = send(fd, he, sizeof(struct sanlk_host_event), 0);
+	rv = send_data(fd, he, sizeof(struct sanlk_host_event), 0);
 	if (rv < 0) {
 		rv = -errno;
 		goto out;
@@ -911,7 +938,7 @@ int sanlock_set_event(const char *ls_name, struct sanlk_host_event *he, uint32_t
 
 	memset(&h, 0, sizeof(h));
 
-	rv = recv(fd, &h, sizeof(h), MSG_WAITALL);
+	rv = recv_data(fd, &h, sizeof(h), MSG_WAITALL);
 	if (rv < 0) {
 		rv = -errno;
 		goto out;
@@ -940,7 +967,7 @@ int sanlock_get_event(int reg_fd, GNUC_UNUSED uint32_t flags, struct sanlk_host_
 	 * get events until we return -EAGAIN to indicate there are no more.
 	 */
 
-	rv = recv(reg_fd, &cb, sizeof(cb), MSG_DONTWAIT);
+	rv = recv_data(reg_fd, &cb, sizeof(cb), MSG_DONTWAIT);
 	if (rv < 0)
 		return -errno;
 
@@ -1012,7 +1039,7 @@ int sanlock_version(uint32_t flags, uint32_t *version, uint32_t *proto)
 
 	memset(&h, 0, sizeof(h));
 
-	rv = recv(fd, &h, sizeof(h), MSG_WAITALL);
+	rv = recv_data(fd, &h, sizeof(h), MSG_WAITALL);
 	if (rv < 0) {
 		rv = -errno;
 		goto out;
@@ -1055,13 +1082,13 @@ int sanlock_killpath(int sock, uint32_t flags, const char *path, char *args)
 	if (rv < 0)
 		return rv;
 
-	rv = send(sock, path_max, SANLK_HELPER_PATH_LEN, 0);
+	rv = send_data(sock, path_max, SANLK_HELPER_PATH_LEN, 0);
 	if (rv < 0) {
 		rv = -errno;
 		goto out;
 	}
 
-	rv = send(sock, args_max, SANLK_HELPER_ARGS_LEN, 0);
+	rv = send_data(sock, args_max, SANLK_HELPER_ARGS_LEN, 0);
 	if (rv < 0) {
 		rv = -errno;
 		goto out;
@@ -1125,27 +1152,27 @@ int sanlock_acquire(int sock, int pid, uint32_t flags, int res_count,
 
 	for (i = 0; i < res_count; i++) {
 		res = res_args[i];
-		rv = send(fd, res, sizeof(struct sanlk_resource), 0);
+		rv = send_data(fd, res, sizeof(struct sanlk_resource), 0);
 		if (rv < 0) {
 			rv = -1;
 			goto out;
 		}
 
-		rv = send(fd, res->disks, sizeof(struct sanlk_disk) * res->num_disks, 0);
+		rv = send_data(fd, res->disks, sizeof(struct sanlk_disk) * res->num_disks, 0);
 		if (rv < 0) {
 			rv = -1;
 			goto out;
 		}
 	}
 
-	rv = send(fd, &opt, sizeof(struct sanlk_options), 0);
+	rv = send_data(fd, &opt, sizeof(struct sanlk_options), 0);
 	if (rv < 0) {
 		rv = -1;
 		goto out;
 	}
 
 	if (opt.len) {
-		rv = send(fd, opt_in->str, opt.len, 0);
+		rv = send_data(fd, opt_in->str, opt.len, 0);
 		if (rv < 0) {
 			rv = -1;
 			goto out;
@@ -1196,7 +1223,7 @@ int sanlock_inquire(int sock, int pid, uint32_t flags, int *res_count,
 
 	memset(&h, 0, sizeof(h));
 
-	rv = recv(fd, &h, sizeof(h), MSG_WAITALL);
+	rv = recv_data(fd, &h, sizeof(h), MSG_WAITALL);
 	if (rv != sizeof(h)) {
 		rv = -1;
 		goto out;
@@ -1214,7 +1241,7 @@ int sanlock_inquire(int sock, int pid, uint32_t flags, int *res_count,
 		goto out;
 	}
 
-	rv = recv(fd, reply_data, len, MSG_WAITALL);
+	rv = recv_data(fd, reply_data, len, MSG_WAITALL);
 	if (rv != len) {
 		free(reply_data);
 		rv = -1;
@@ -1264,7 +1291,7 @@ int sanlock_convert(int sock, int pid, uint32_t flags, struct sanlk_resource *re
 	if (rv < 0)
 		goto out;
 
-	rv = send(fd, res, sizeof(struct sanlk_resource), 0);
+	rv = send_data(fd, res, sizeof(struct sanlk_resource), 0);
 	if (rv < 0) {
 		rv = -errno;
 		goto out;
@@ -1310,7 +1337,7 @@ int sanlock_release(int sock, int pid, uint32_t flags, int res_count,
 		goto out;
 
 	for (i = 0; i < res_count; i++) {
-		rv = send(fd, res_args[i], sizeof(struct sanlk_resource), 0);
+		rv = send_data(fd, res_args[i], sizeof(struct sanlk_resource), 0);
 		if (rv < 0) {
 			rv = -1;
 			goto out;
@@ -1343,13 +1370,13 @@ int sanlock_request(uint32_t flags, uint32_t force_mode,
 	if (rv < 0)
 		goto out;
 
-	rv = send(fd, res, sizeof(struct sanlk_resource), 0);
+	rv = send_data(fd, res, sizeof(struct sanlk_resource), 0);
 	if (rv < 0) {
 		rv = -errno;
 		goto out;
 	}
 
-	rv = send(fd, res->disks, sizeof(struct sanlk_disk) * res->num_disks, 0);
+	rv = send_data(fd, res->disks, sizeof(struct sanlk_disk) * res->num_disks, 0);
 	if (rv < 0) {
 		rv = -errno;
 		goto out;
@@ -1388,7 +1415,7 @@ int sanlock_examine(uint32_t flags, struct sanlk_lockspace *ls,
 	if (rv < 0)
 		goto out;
 
-	rv = send(fd, data, datalen, 0);
+	rv = send_data(fd, data, datalen, 0);
 	if (rv < 0) {
 		rv = -errno;
 		goto out;
@@ -1418,13 +1445,13 @@ int sanlock_set_lvb(uint32_t flags, struct sanlk_resource *res, char *lvb, int l
 	if (rv < 0)
 		return rv;
 
-	rv = send(fd, res, sizeof(struct sanlk_resource), 0);
+	rv = send_data(fd, res, sizeof(struct sanlk_resource), 0);
 	if (rv < 0) {
 		rv = -errno;
 		goto out;
 	}
 
-	rv = send(fd, lvb, lvblen, 0);
+	rv = send_data(fd, lvb, lvblen, 0);
 	if (rv < 0) {
 		rv = -1;
 		goto out;
@@ -1456,7 +1483,7 @@ int sanlock_get_lvb(uint32_t flags, struct sanlk_resource *res, char *lvb, int l
 	if (rv < 0)
 		return rv;
 
-	rv = send(fd, res, sizeof(struct sanlk_resource), 0);
+	rv = send_data(fd, res, sizeof(struct sanlk_resource), 0);
 	if (rv < 0) {
 		rv = -1;
 		goto out;
@@ -1466,7 +1493,7 @@ int sanlock_get_lvb(uint32_t flags, struct sanlk_resource *res, char *lvb, int l
 
 	memset(&h, 0, sizeof(h));
 
-	rv = recv(fd, &h, sizeof(h), MSG_WAITALL);
+	rv = recv_data(fd, &h, sizeof(h), MSG_WAITALL);
 	if (rv != sizeof(h)) {
 		rv = -1;
 		goto out;
@@ -1484,7 +1511,7 @@ int sanlock_get_lvb(uint32_t flags, struct sanlk_resource *res, char *lvb, int l
 		goto out;
 	}
 
-	rv = recv(fd, reply_data, len, MSG_WAITALL);
+	rv = recv_data(fd, reply_data, len, MSG_WAITALL);
 	if (rv != len) {
 		free(reply_data);
 		rv = -1;
@@ -1972,3 +1999,79 @@ int sanlock_str_to_lockspace(char *str, struct sanlk_lockspace *ls)
 	return 0;
 }
 
+const char *sanlock_strerror(int rv)
+{
+	switch (rv) {
+	case SANLK_ERROR:
+		return "General error";
+	case SANLK_AIO_TIMEOUT:
+		return "IO timeout";
+	case SANLK_WD_ERROR:
+		return "Watchdog device error";
+	case SANLK_DBLOCK_READ:
+		return "Lease read error in dblock";
+	case SANLK_DBLOCK_WRITE:
+		return "Lease write error in dblock";
+	case SANLK_DBLOCK_MBAL:
+		return "Lease was acquired by another host in current ballot";
+	case SANLK_DBLOCK_LVER:
+		return "Lease was acquired by another host in new ballot";
+	case SANLK_DBLOCK_CHECKSUM:
+		return "Lease checksum error in dblock";
+	case SANLK_LEADER_READ:
+		return "Lease read error in leader";
+	case SANLK_LEADER_WRITE:
+		return "Lease write error in leader";
+	case SANLK_LEADER_DIFF:
+		return "Lease read inconsistent";
+	case SANLK_LEADER_MAGIC:
+		return "Lease does not exist on storage";
+	case SANLK_LEADER_VERSION:
+		return "Lease format version on storage is not recognized";
+	case SANLK_LEADER_SECTORSIZE:
+		return "Lease sector size is inconsistent";
+	case SANLK_LEADER_LOCKSPACE:
+		return "Lease lockspace name is incorrect";
+	case SANLK_LEADER_RESOURCE:
+		return "Lease resource name is incorrect";
+	case SANLK_LEADER_NUMHOSTS:
+		return "Lease num_hosts is incorrect";
+	case SANLK_LEADER_CHECKSUM:
+		return "Lease checksum error in leader";
+	case SANLK_ACQUIRE_LOCKSPACE:
+		return "Lease lockspace is not found";
+	case SANLK_ACQUIRE_IDDISK:
+		return "Lease lockspace disk cannot be opened";
+	case SANLK_ACQUIRE_IDLIVE:
+		return "Lease is held by another host";
+	case SANLK_ACQUIRE_OWNED:
+		return "Lease was acquired by another host in other ballot";
+	case SANLK_ACQUIRE_OTHER:
+		return "Lease was acquired by another host in local commit";
+	case SANLK_ACQUIRE_SHRETRY:
+		return "Lease is held by another host for shared acquire";
+	case SANLK_ACQUIRE_OWNED_RETRY:
+		return "Lease is owned by another host";
+	case SANLK_RELEASE_LVER:
+		return "Lease release version is incorrect";
+	case SANLK_RELEASE_OWNER:
+		return "Lease release owner is incorrect";
+	case SANLK_RENEW_OWNER:
+		return "Lease renew owner is incorrect";
+	case SANLK_RENEW_DIFF:
+		return "Lease renew data has changed";
+	case SANLK_HOSTID_BUSY:
+		return "Lease host ID is being used by another host";
+	case SANLK_REQUEST_MAGIC:
+		return "Lease request block has invalid data";
+	case SANLK_REQUEST_VERSION:
+		return "Lease request block has invalid version";
+	case SANLK_REQUEST_OLD:
+		return "Lease request has newer lease version";
+	case SANLK_REQUEST_LVER:
+		return "Lease request block has newer version";
+	default:
+		return "Unknown error";
+	};
+}
+
