@@ -35,7 +35,8 @@ int delta_lease_renew(struct task *task,
 		      int *read_result,
 		      int log_renewal_level,
                       struct leader_record *leader_last,
-                      struct leader_record *leader_ret);
+                      struct leader_record *leader_ret,
+		      int *rd_ms, int *wr_ms);
 
 int delta_lease_release(struct task *task,
                         struct space *sp,
