@@ -21,10 +21,10 @@ int majority_disks(int num_disks, int num);
  */
 
 int write_iobuf(int fd, uint64_t offset, char *iobuf, int iobuf_len,
-		struct task *task, int ioto);
+		struct task *task, int ioto, int *wr_ms);
 
 int read_iobuf(int fd, uint64_t offset, char *iobuf, int iobuf_len,
-	       struct task *task, int ioto);
+	       struct task *task, int ioto, int *rd_ms);
 
 int read_iobuf_reap(int fd, uint64_t offset, char *iobuf, int iobuf_len,
 		    struct task *task, uint32_t ioto_msec);
