@@ -531,6 +531,44 @@ static void close_event_fds(struct space *sp)
 	pthread_mutex_unlock(&sp->mutex);
 }
 
+/*
+ * if delta_result is success:
+ * new record saving last_success (timestamp in renewal), rd_ms, wr_ms
+ * if delta_result is timeout:
+ * increment next_timeouts in prev record
+ * if delta_result is failure:
+ * increment next_errors in prev record
+ */
+
+static void save_renewal_history(struct space *sp, int delta_result,
+				 uint64_t last_success, int rd_ms, int wr_ms)
+{
+	struct renewal_history *hi;
+
+	if (!sp->renewal_history_size || !sp->renewal_history)
+		return;
+
+	if (delta_result == SANLK_OK) {
+		hi = &sp->renewal_history[sp->renewal_history_next];
+
+		hi->timestamp = last_success;
+		hi->read_ms = rd_ms;
+		hi->write_ms = wr_ms;
+
+		sp->renewal_history_prev = sp->renewal_history_next;
+		sp->renewal_history_next++;
+		if (sp->renewal_history_next >= sp->renewal_history_size)
+			sp->renewal_history_next = 0;
+	} else {
+		hi = &sp->renewal_history[sp->renewal_history_prev];
+
+		if (delta_result == SANLK_AIO_TIMEOUT)
+			hi->next_timeouts++;
+		else
+			hi->next_errors++;
+	}
+}
+
 /*
  * This thread must not be stopped unless all pids that may be using any
  * resources in it are dead/gone.  (The USED flag in the lockspace represents
@@ -551,6 +589,7 @@ static void *lockspace_thread(void *arg_in)
 	int rv, delta_length, renewal_interval = 0;
 	int id_renewal_seconds, id_renewal_fail_seconds;
 	int acquire_result, delta_result, read_result;
+	int rd_ms, wr_ms;
 	int opened = 0;
 	int stop = 0;
 	int wd_con;
@@ -642,6 +681,8 @@ static void *lockspace_thread(void *arg_in)
 	sp->lease_status.renewal_last_attempt = delta_begin;
 	if (delta_result == SANLK_OK)
 		sp->lease_status.renewal_last_success = last_success;
+	/* First renewal entry shows the acquire time with 0 latencies. */
+	save_renewal_history(sp, delta_result, last_success, 0, 0);
 	pthread_mutex_unlock(&sp->mutex);
 
 	if (acquire_result < 0)
@@ -685,7 +726,8 @@ static void *lockspace_thread(void *arg_in)
 						 sp->space_name, bitmap, &extra,
 						 delta_result, &read_result,
 						 log_renewal_level,
-						 &leader, &leader);
+						 &leader, &leader,
+						 &rd_ms, &wr_ms);
 		delta_length = monotime() - delta_begin;
 
 		if (delta_result == SANLK_OK) {
@@ -714,7 +756,6 @@ static void *lockspace_thread(void *arg_in)
 			sp->lease_status.renewal_read_count++;
 		}
 
-
 		/*
 		 * pet the watchdog
 		 * (don't update on thread_stop because it's probably unlinked)
@@ -723,6 +764,7 @@ static void *lockspace_thread(void *arg_in)
 		if (delta_result == SANLK_OK && !sp->thread_stop)
 			update_watchdog(sp, last_success, id_renewal_fail_seconds);
 
+		save_renewal_history(sp, delta_result, last_success, rd_ms, wr_ms);
 		pthread_mutex_unlock(&sp->mutex);
 
 
@@ -819,6 +861,14 @@ int add_lockspace_start(struct sanlk_lockspace *ls, uint32_t io_timeout, struct
 	for (i = 0; i < MAX_EVENT_FDS; i++)
 		sp->event_fds[i] = -1;
 
+	if (com.renewal_history_size) {
+		sp->renewal_history = malloc(sizeof(struct renewal_history) * com.renewal_history_size);
+		if (sp->renewal_history) {
+			sp->renewal_history_size = com.renewal_history_size;
+			memset(sp->renewal_history, 0, sizeof(struct renewal_history) * com.renewal_history_size);
+		}
+	}
+
 	pthread_mutex_lock(&spaces_mutex);
 
 	/* search all lists for an identical lockspace */
