@@ -73,6 +73,7 @@ struct thread_pool {
 
 /* priorities are LOG_* from syslog.h */
 int log_logfile_priority = LOG_WARNING;
+int log_logfile_use_utc = 0;
 int log_syslog_priority = LOG_ERR;
 int log_stderr_priority = -1; /* -D sets this to LOG_DEBUG */
 
@@ -1218,6 +1219,7 @@ static void process_connection(int ci)
 	case SM_CMD_SHUTDOWN:
 	case SM_CMD_STATUS:
 	case SM_CMD_HOST_STATUS:
+	case SM_CMD_RENEWAL:
 	case SM_CMD_LOG_DUMP:
 	case SM_CMD_GET_LOCKSPACES:
 	case SM_CMD_GET_HOSTS:
@@ -1807,6 +1809,7 @@ static void print_usage(void)
 	printf("  -D            no fork and print all logging to stderr\n");
 	printf("  -Q 0|1        quiet error messages for common lock contention (%d)\n", DEFAULT_QUIET_FAIL);
 	printf("  -R 0|1        renewal debugging, log debug info about renewals (0)\n");
+	printf("  -H <num>      renewal history size (%d)\n", DEFAULT_RENEWAL_HISTORY_SIZE);
 	printf("  -L <pri>      write logging at priority level and up to logfile (3 LOG_ERR)\n");
 	printf("                (use -1 for none)\n");
 	printf("  -S <pri>      write logging at priority level and up to syslog (3 LOG_ERR)\n");
@@ -1827,11 +1830,12 @@ static void print_usage(void)
 	printf("sanlock client status [-D] [-o p|s]\n");
 	printf("sanlock client gets [-h 0|1]\n");
 	printf("sanlock client host_status -s LOCKSPACE [-D]\n");
+	printf("sanlock client renewal -s LOCKSPACE\n");
 	printf("sanlock client set_event -s LOCKSPACE -i <host_id> [-g gen] -e <event> -d <data>\n");
 	printf("sanlock client set_config -s LOCKSPACE [-u 0|1] [-O 0|1]\n");
 	printf("sanlock client log_dump\n");
 	printf("sanlock client shutdown [-f 0|1] [-w 0|1]\n");
-	printf("sanlock client init -s LOCKSPACE | -r RESOURCE\n");
+	printf("sanlock client init -s LOCKSPACE | -r RESOURCE [-z 0|1]\n");
 	printf("sanlock client read -s LOCKSPACE | -r RESOURCE\n");
 	printf("sanlock client align -s LOCKSPACE\n");
 	printf("sanlock client add_lockspace -s LOCKSPACE\n");
@@ -1848,18 +1852,18 @@ static void print_usage(void)
 	printf("sanlock direct <action> [-a 0|1] [-o 0|1]\n");
 	printf("sanlock direct init -s LOCKSPACE | -r RESOURCE\n");
 	printf("sanlock direct read_leader -s LOCKSPACE | -r RESOURCE\n");
-	printf("sanlock direct dump <path>[:<offset>]\n");
+	printf("sanlock direct dump <path>[:<offset>[:<size>]]\n");
 	printf("\n");
 	printf("LOCKSPACE = <lockspace_name>:<host_id>:<path>:<offset>\n");
 	printf("  <lockspace_name>	name of lockspace\n");
 	printf("  <host_id>		local host identifier in lockspace\n");
-	printf("  <path>		disk to storage reserved for leases\n");
+	printf("  <path>		path to storage reserved for leases\n");
 	printf("  <offset>		offset on path (bytes)\n");
 	printf("\n");
 	printf("RESOURCE = <lockspace_name>:<resource_name>:<path>:<offset>[:<lver>]\n");
 	printf("  <lockspace_name>	name of lockspace\n");
 	printf("  <resource_name>	name of resource\n");
-	printf("  <path>		disk to storage reserved for leases\n");
+	printf("  <path>		path to storage reserved for leases\n");
 	printf("  <offset>		offset on path (bytes)\n");
 	printf("  <lver>                optional leader version or SH for shared lease\n");
 	printf("\n");
@@ -1935,6 +1939,8 @@ static int read_command_line(int argc, char *argv[])
 			com.action = ACT_STATUS;
 		else if (!strcmp(act, "host_status"))
 			com.action = ACT_HOST_STATUS;
+		else if (!strcmp(act, "renewal"))
+			com.action = ACT_RENEWAL;
 		else if (!strcmp(act, "gets"))
 			com.action = ACT_GETS;
 		else if (!strcmp(act, "log_dump"))
@@ -2051,6 +2057,9 @@ static int read_command_line(int argc, char *argv[])
 		case 'R':
 			com.debug_renew = atoi(optionarg);
 			break;
+		case 'H':
+			com.renewal_history_size = atoi(optionarg);
+			break;
 		case 'L':
 			log_logfile_priority = atoi(optionarg);
 			break;
@@ -2152,6 +2161,9 @@ static int read_command_line(int argc, char *argv[])
 			com.used_set = 1;
 			com.used = atoi(optionarg);
 			break;
+		case 'z':
+			com.clear_arg = 1;
+			break;
 
 		case 'c':
 			begin_command = 1;
@@ -2289,6 +2301,10 @@ static void read_config_file(void)
 			get_val_int(line, &val);
 			log_logfile_priority = val;
 
+		} else if (!strcmp(str, "logfile_use_utc")) {
+			get_val_int(line, &val);
+			log_logfile_use_utc = val;
+
 		} else if (!strcmp(str, "syslog_priority")) {
 			get_val_int(line, &val);
 			log_syslog_priority = val;
@@ -2331,6 +2347,14 @@ static void read_config_file(void)
 			get_val_int(line, &val);
 			com.renewal_read_extend_sec_set = 1;
 			com.renewal_read_extend_sec = val;
+
+		} else if (!strcmp(str, "renewal_history_size")) {
+			get_val_int(line, &val);
+			com.renewal_history_size = val;
+
+		} else if (!strcmp(str, "paxos_debug_all")) {
+			get_val_int(line, &val);
+			com.paxos_debug_all = val;
 		}
 	}
 
@@ -2571,6 +2595,10 @@ static int do_client(void)
 		rv = sanlock_host_status(com.debug, com.lockspace.name);
 		break;
 
+	case ACT_RENEWAL:
+		rv = sanlock_renewal(com.lockspace.name);
+		break;
+
 	case ACT_GETS:
 		rv = do_client_gets();
 		break;
@@ -2640,7 +2668,7 @@ static int do_client(void)
 	case ACT_ACQUIRE:
 		log_tool("acquire pid %d", com.pid);
 		flags |= com.orphan ? SANLK_ACQUIRE_ORPHAN : 0;
-		rv = sanlock_acquire(-1, com.pid, 0, com.res_count, com.res_args, NULL);
+		rv = sanlock_acquire(-1, com.pid, flags, com.res_count, com.res_args, NULL);
 		log_tool("acquire done %d", rv);
 		break;
 
@@ -2740,7 +2768,8 @@ static int do_client(void)
 		else
 			rv = sanlock_write_resource(com.res_args[0],
 						    com.max_hosts,
-						    com.num_hosts, 0);
+						    com.num_hosts,
+						    com.clear_arg ? SANLK_WRITE_CLEAR : 0);
 		log_tool("init done %d", rv);
 		break;
 
@@ -3026,7 +3055,7 @@ static int do_direct_init(void)
 		}
 
 		rv = direct_write_resource(&main_task, com.res_args[0],
-					   com.max_hosts, com.num_hosts);
+					   com.max_hosts, com.num_hosts, com.clear_arg);
 	}
 
 	log_tool("init done %d", rv);
@@ -3189,6 +3218,8 @@ int main(int argc, char *argv[])
 	com.quiet_fail = DEFAULT_QUIET_FAIL;
 	com.renewal_read_extend_sec_set = 0;
 	com.renewal_read_extend_sec = 0;
+	com.renewal_history_size = DEFAULT_RENEWAL_HISTORY_SIZE;
+	com.paxos_debug_all = 0;
 
 	if (getgrnam("sanlock") && getpwnam("sanlock")) {
 		com.uname = (char *)"sanlock";
