@@ -9,35 +9,34 @@
 #ifndef __SANLOCK_DIRECT_H__
 #define __SANLOCK_DIRECT_H__
 
-/*
- * Use io_timeout_sec = 0 for default value
- */
-
-int sanlock_direct_read_id(struct sanlk_lockspace *ls,
-                           uint64_t *timestamp,
-                           uint64_t *owner_id,
-                           uint64_t *owner_generation,
-                           int use_aio,
-			   int io_timeout_sec);
-
-int sanlock_direct_live_id(struct sanlk_lockspace *ls,
-                           uint64_t *timestamp,
-                           uint64_t *owner_id,
-                           uint64_t *owner_generation,
-                           int *live,
-                           int use_aio,
-			   int io_timeout_sec);
-
 /*
  * Use max_hosts = 0 for default value.
  * Use num_hosts = 0 for default value.
  * Provide either lockspace or resource, not both
+ *
+ * (Old api, see write_lockspace/resource)
  */
 
 int sanlock_direct_init(struct sanlk_lockspace *ls,
                         struct sanlk_resource *res,
                         int max_hosts, int num_hosts, int use_aio);
 
+/*
+ * write a lockspace to disk
+ * (also see sanlock_write_lockspace)
+ */
+
+int sanlock_direct_write_lockspace(struct sanlk_lockspace *ls, int max_hosts,
+				   uint32_t flags, uint32_t io_timeout);
+
+/*
+ * format a resource lease area on disk
+ * (also see sanlock_write_resource)
+ */
+
+int sanlock_direct_write_resource(struct sanlk_resource *res,
+				  int max_hosts, int num_hosts, uint32_t flags);
+
 /*
  * Returns the alignment in bytes required by sanlock_direct_init()
  * (1MB for disks with 512 sectors, 8MB for disks with 4096 sectors)
